@@ -481,7 +481,7 @@ class SIPMessage():
       
 class SIPClient():
   def __init__(self, server, port, username, password, myIP=None, myPort=5060, callCallback=None):
-    self.NSD = True
+    self.NSD = False
     self.server = server
     self.port = port
     self.hostname = socket.gethostname()
@@ -569,6 +569,9 @@ class SIPClient():
       debug("TODO: Add 400 Error on non processable request")
   
   def start(self):
+    if self.NSD == True:
+      raise RuntimeError("Attempted to start already started SIPClient")
+    self.NSD = True
     self.s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
     #self.out = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
     self.s.bind((self.myIP, self.myPort))
@@ -580,33 +583,41 @@ class SIPClient():
      
   def stop(self):
     self.NSD = False
-    self.registerThread.cancel()
-    self.deregister()
-    self.s.close()
-    self.out.close()
+    if self.registerThread:
+      self.registerThread.cancel()
+      self.deregister()
+    self._close_sockets()
+
+  def _close_sockets(self):
+    if hasattr(self, 's') and self.s:
+        self.s.close()
+    if hasattr(self, 'out') and self.out:
+        self.out.close()
     
   def genCallID(self):
     return hashlib.sha256(str(self.callID.next()).encode('utf8')).hexdigest()[0:32]+"@"+self.myIP+":"+str(self.myPort)
   
   def genTag(self):
-    while True:
+    while True:  # Keep as True instead of NSD so it can generate a tag on deregister.
       tag = hashlib.md5(str(random.randint(1, 4294967296)).encode('utf8')).hexdigest()[0:8]
       if tag not in self.tags:
         self.tags.append(tag)
         return tag
   
   def genSIPVersionNotSupported(self, request):
-    regRequest = "SIP/2.0 505 SIP Version Not Supported\r\n"
-    regRequest += "Via: SIP/2.0/UDP "+request.headers['Via']['address'][0]+":"+request.headers['Via']['address'][1]+";branch="+request.headers['Via']['branch']+"\r\n"
-    regRequest += "From: "+request.headers['From']['raw']+";tag="+request.headers['From']['tag']+"\r\n"
-    regRequest += "To: "+request.headers['To']['raw']+";tag="+self.genTag()+"\r\n"
-    regRequest += "Call-ID: "+request.headers['Call-ID']+"\r\n"
-    regRequest += "CSeq: "+request.headers['CSeq']['check']+" "+request.headers['CSeq']['method']+"\r\n"
-    regRequest += "Contact: "+request.headers['Contact']+"\r\n" #TODO: Add Supported
-    regRequest += "User-Agent: pyVoIP """+pyVoIP.__version__+"\r\n"
-    regRequest += "Warning: 399 GS \"Unable to accept call\"\r\n"
-    regRequest += "Allow: "+(", ".join(pyVoIP.SIPCompatibleMethods))+"\r\n"
-    regRequest += "Content-Length: 0\r\n\r\n"
+    response = "SIP/2.0 505 SIP Version Not Supported\r\n"
+    response += "Via: SIP/2.0/UDP "+request.headers['Via']['address'][0]+":"+request.headers['Via']['address'][1]+";branch="+request.headers['Via']['branch']+"\r\n"
+    response += "From: "+request.headers['From']['raw']+";tag="+request.headers['From']['tag']+"\r\n"
+    response += "To: "+request.headers['To']['raw']+";tag="+self.genTag()+"\r\n"
+    response += "Call-ID: "+request.headers['Call-ID']+"\r\n"
+    response += "CSeq: "+request.headers['CSeq']['check']+" "+request.headers['CSeq']['method']+"\r\n"
+    response += "Contact: "+request.headers['Contact']+"\r\n" #TODO: Add Supported
+    response += "User-Agent: pyVoIP """+pyVoIP.__version__+"\r\n"
+    response += "Warning: 399 GS \"Unable to accept call\"\r\n"
+    response += "Allow: "+(", ".join(pyVoIP.SIPCompatibleMethods))+"\r\n"
+    response += "Content-Length: 0\r\n\r\n"
+
+    return response
   
   def genAuthorization(self, request):
     HA1 = hashlib.md5(self.username.encode('utf8')+b':'+request.authentication['realm'].encode('utf8')+b':'+self.password.encode('utf8')).hexdigest().encode('utf8')
@@ -632,7 +643,7 @@ class SIPClient():
     regRequest += "\r\nCSeq: "+str(self.registerCounter.next())+" REGISTER"
     regRequest +="\r\nExpires: 300\r\nAllow: "+(", ".join(pyVoIP.SIPCompatibleMethods))+"\r\nUser-Agent: pyVoIP """+pyVoIP.__version__+"\r\n"
     regRequest += 'Authorization: Digest username="'+self.username
-    regRequest += '",realm="+realm+",nonce="'+nonce
+    regRequest += '",realm="'+realm+'",nonce="'+nonce
     regRequest += '",uri="sip:'+self.server
     regRequest += ';transport=UDP",response="'+str(response, 'utf8')
     regRequest += '",algorithm=MD5\r\n'+"Content-Length: 0\r\n\r\n"
@@ -823,7 +834,7 @@ class SIPClient():
     nonce = response.authentication['nonce']
     realm = response.authentication['realm']
     auth = 'Authorization: Digest username="'+self.username
-    auth += '",realm="+realm+",nonce="'+nonce
+    auth += '",realm="'+realm+'",nonce="'+nonce
     auth += '",uri="sip:'+self.server
     auth += ';transport=UDP",response="'+str(authhash, 'utf8')
     auth += '",algorithm=MD5\r\n'
